
package HostingMaster;

use strict;
use IO::Socket;
use Digest::MD5 qw(md5_base64);
use vars qw(%default);

%default =
    (
     'remoteport' => 31337,
     'host' => 'localhost',
     'username' => $ENV{'USER'},
     'password' => undef,
     );

sub new {
    my $class = shift;
    my %settings = ref $_[0] ? %{$_[0]} : @_;
    my $self = { %default };
    foreach( keys %settings ) {
	$self->{$_} = $settings{$_};
    }
    return bless $self, $class;
}


sub connect {
    my ( $self, $host, $port ) = @_;
    $host ||= $self->{'host'};
    $port ||= $self->{'remoteport'};

    return $self->{'socket'} = 
      IO::Socket::INET->new( 'Proto' => 'tcp',
			     'PeerAddr' => $host,
			     'PeerPort' => $port )
	  or die "Connection setup failed";
}


sub prepare {
    my ( $self, @cmd ) = @_;
    my $user = $self->{'username'} || $ENV{'USER'};
    my $pass = $self->{'password'} || '';
    my $sock = $self->{'socket'} || $self->connect();

    $self->{'command'} = "@cmd";
    foreach( qw( username command session sessionid ) ) {
	print $sock "$_: $self->{$_}\n" if $self->{$_};
    }
    print $sock "\n";

    chomp( my $in = <$sock> );
    my ( $challenge, $seed ) = $in =~/^Challenge:\s+(\S+)\s*(\S*)$/i;
    if( $challenge ) {
	$pass ||= $self->{'password_callback'}->($self)
	    if $self->{'password_callback'};

	$pass = crypt $pass, "\$1\$$seed";
	$pass =~ s/.*\$//;
	$pass = md5_base64 "$pass$challenge";

	print $sock "Response: $pass\n\n";
	chomp( $in = <$sock> );

	die "$in\n" unless $in =~ /^ok\b/i;
	$in = '';
    }

    $sock->autoflush(1);

    return $in ? ( $sock, $in ) : $sock;
}

sub mediate {
    my ( $self, $input, $output, $sock, @output ) = @_;

    $sock ||= $self->{'socket'} or die "Missing socket\n";
    $input ||= $self->{'input'} || \*STDIN;
    $output ||= $self->{'output'} || \*STDOUT;

    if( my $pid = fork() ) {
	print $output @output if @output;

	my $c;
	while( !eof($sock) and defined( $c = getc $sock )
	       and ( $c ne "\04" ) ) {
	    print $output $c unless $c eq '';
	}

	kill 15, $pid;
	waitpid $pid, 0;
    } else {
	while( <$input> ) {
	    print $sock $_;
	}
	exit 0;
    }

    return $sock;
}

sub run {
    my ( $self, @cmd ) = @_;
    $self->connect();
    my ( $sock, @output ) = $self->prepare( @cmd );
    $self->mediate( undef, undef, $sock, @output );
    close $sock;
}

1;
